% Example-45: token selection from multiple input places
% Reggie.davidrajuh@uis.no  (c) 15 Nov 2017

function [fire, transition] = COMMON_PRE (transition)

% "tColor1" and "tColor2" have their own specific 
%   pre-processor files, hence just leave
if ismember(transition.name, {'tColor1', 'tColor2'})
    fire = 1; return
end

% FREEZE: Do not allow "tA_and_X", "tA_or_B", "tA_or_B_Preferred"
%               to fire until ct = 50
ct = current_time();
if le(ct, 50), fire = 0; return, end

% conditions for "tA_and_X", "tA_or_B", "tA_or_B_Preferred" 
if strcmpi(transition.name, 'tA_and_X')
    tokID1 = tokenAnyColor('pAB', 1, {'A'});
    tokID2 = tokenAnyColor('pXY', 1, {'X'});
    selected_tokens = [tokID1 tokID2];    % tokens to be removed
    fire = all(selected_tokens); % must have both "A" and "X"
       
elseif strcmpi(transition.name, 'tA_or_X')
    % let tA_and_X consume first, starting at time = 50
    % when time == 70, allow tA_or_X to function
    if le(ct, 70), fire = 0;return, end 
    
    tokID1 = tokenAnyColor ('pAB', 1, {'A'});
    tokID2 = tokenAnyColor ('pXY', 1, {'X'});
    selected_tokens = [tokID1 tokID2];    % tokens to be removed
    fire = any(selected_tokens); % must be at least "A" or "X" 

elseif strcmpi(transition.name, 'tA_or_X_Preferred')
    % let tA_and_X consume first, starting at time = 50
    % then let tA_or_X to consume, starting at time = 70
    % finally, when time == 90, allow tA_or_X_Preferred to function
    if le(ct, 90), fire = 0; return, end % let "tA_and_X" & "tA_or_X" consume first

    tokID1 = tokenAnyColor('pAB', 1, {'A'});
    tokID2 = tokenAnyColor('pXY', 1, {'X'});
    selected_tokens = [tokID1 tokID2];    % tokens to be removed
    fire = 1;
else % not possible
end
transition.selected_tokens = selected_tokens; 
